package com.vulnit.projects.xss;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.List;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLException;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.HttpVersion;
import org.apache.http.ProtocolException;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.NTCredentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.auth.params.AuthPNames;
import org.apache.http.client.CircularRedirectException;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.HttpClient;
import org.apache.http.client.RedirectStrategy;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.params.AuthPolicy;
import org.apache.http.client.params.ClientPNames;
import org.apache.http.client.utils.URIUtils;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.conn.ssl.X509HostnameVerifier;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.RedirectLocations;
import org.apache.http.impl.client.RequestWrapper;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.ExecutionContext;
import org.apache.http.protocol.HttpContext;

/**
 * 
 * @author rmillet
 * 
 */
public final class HttpClientManager {
	private static final String DEFAULT_USER_AGENT = "VulnIT XSS Scanner";
	private static HttpClientManager instance = new HttpClientManager();

	private final CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
	private final List<String> authPref = new ArrayList<String>();
	private DefaultHttpClient httpClient = null;

	private String cookies = null;

	private String referer = null;

	private String userAgent = DEFAULT_USER_AGENT;

	private int port = 443;

	private int timeout = 5000;

	/**
	 * Default private constructor
	 */
	private HttpClientManager() {
	}

	/**
	 * 
	 * @return The instance
	 */
	public static HttpClientManager getInstance() {
		return instance;
	}

	/**
	 * 
	 * @return The {@link HttpClient}
	 */
	private HttpClient getClient() {
		if (this.httpClient != null) {
			return this.httpClient;
		}
		HttpParams params = new BasicHttpParams();
		HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
		HttpProtocolParams.setUserAgent(params, this.userAgent);
		HttpConnectionParams.setConnectionTimeout(params, this.timeout);
		HttpConnectionParams.setSoTimeout(params, this.timeout);

		SchemeRegistry schemeRegistry = new SchemeRegistry();
		try {
			SSLContext sslContext = SSLContext.getInstance(SSLSocketFactory.TLS);
			sslContext.init(null, new TrustManager[] {
			// Scheme to accept any certificate
					new X509TrustManager() {
						@Override
						public X509Certificate[] getAcceptedIssuers() {
							return null;
						}

						@Override
						public void checkClientTrusted(X509Certificate[] certs, String authType) {
						}

						@Override
						public void checkServerTrusted(X509Certificate[] certs, String authType) {
						}
					} }, new SecureRandom());

			X509HostnameVerifier hostnameVerifier = new X509HostnameVerifier() {
				@Override
				public boolean verify(String hostname, SSLSession session) {
					return true;
				}

				@Override
				public void verify(String host, String[] cns, String[] subjectAlts) throws SSLException {
				}

				@Override
				public void verify(String host, X509Certificate cert) throws SSLException {
				}

				@Override
				public void verify(String host, SSLSocket ssl) throws IOException {
				}
			};
			SSLSocketFactory sslSocketFactory = new SSLSocketFactory(sslContext, hostnameVerifier);
			schemeRegistry.register(new Scheme("https", this.port, sslSocketFactory));
			schemeRegistry.register(new Scheme("http", this.port, PlainSocketFactory
					.getSocketFactory()));
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (KeyManagementException e) {
			e.printStackTrace();
		}

		ThreadSafeClientConnManager connectionManager = new ThreadSafeClientConnManager(
				schemeRegistry);
		connectionManager.setMaxTotal(10);

		this.httpClient = new DefaultHttpClient(connectionManager, params);
		this.httpClient.setRedirectStrategy(new SameHostRedirectStrategy());
		this.httpClient.setCredentialsProvider(this.credentialsProvider);
		this.httpClient.getParams().setParameter(AuthPNames.TARGET_AUTH_PREF, this.authPref);
		return this.httpClient;
	}

	/**
	 * Execute the request with all parameters defined (cookies...)
	 * 
	 * @param request
	 *            The request to execute
	 * @return An {@link HttpEntity} of the response or <code>null</code> if there was an error
	 * @throws IOException
	 *             on execution error
	 */
	public HttpEntity execute(HttpUriRequest request) throws IOException {

		if (this.cookies != null) {
			request.addHeader("Cookie", this.cookies);
		}

		if (this.referer != null) {
			request.addHeader("Referer", this.referer);
		}

		return this.getClient().execute(request).getEntity();
	}

	/**
	 * 
	 * @param cookies
	 *            The Cookies
	 */
	public void setCookies(String cookies) {
		this.cookies = cookies;
	}

	/**
	 * 
	 * @param port
	 *            The port
	 */
	public void setPort(int port) {
		this.port = port;
	}

	/**
	 * 
	 * @param referer
	 *            The Referer
	 */
	public void setReferer(String referer) {
		this.referer = referer;
	}

	/**
	 * 
	 * @param userAgent
	 *            The User-Agent
	 * @see {@link #DEFAULT_USER_AGENT} for default value
	 */
	public void setUserAgent(String userAgent) {
		if (userAgent == null) {
			this.userAgent = DEFAULT_USER_AGENT;
		} else {
			this.userAgent = userAgent;
		}
		httpClient = null;
	}

	/**
	 * 
	 * @param timeout
	 *            Timeout (milliseconds)
	 */
	public void setTimeout(int timeout) {
		this.timeout = timeout;
		httpClient = null;
	}

	/**
	 * Set credentials
	 * 
	 * @param usernamePassword
	 *            username:password for basic and digest<br/>
	 *            domain/username:password for ntlm<br/>
	 *            <code>null</code> for spnego
	 * @param authType
	 *            NTLM, Basic, Digest, SPNEGO
	 */
	public void setCredentials(String usernamePassword, String authType) {
		String type = authType.toLowerCase();
		this.authPref.clear();

		if ("basic".equals(type)) {
			this.credentialsProvider.setCredentials(new AuthScope(AuthScope.ANY_HOST,
					AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthPolicy.BASIC),
					new UsernamePasswordCredentials(usernamePassword));
			this.authPref.add(AuthPolicy.BASIC);
		} else if ("digest".equals(type)) {
			this.credentialsProvider.setCredentials(new AuthScope(AuthScope.ANY_HOST,
					AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthPolicy.DIGEST),
					new UsernamePasswordCredentials(usernamePassword));
			this.authPref.add(AuthPolicy.DIGEST);
		} else if ("ntlm".equals(type)) {
			// We can't NTCredentials(String) because it create a null workstation which causes a
			// null-pointer exception
			String username = "";
			String password = "";
			String domain = "";

			int colonPos = usernamePassword.indexOf(':');
			if (colonPos > 0) {
				username = usernamePassword.substring(0, colonPos);
				password = usernamePassword.substring(colonPos + 1);
				int slashPos = usernamePassword.indexOf('/');
				if (slashPos <= 0) {
					slashPos = usernamePassword.indexOf('\\');
				}
				if (slashPos > 0) {
					domain = username.substring(0, slashPos);
					username = username.substring(slashPos + 1);
				}
			}
			this.credentialsProvider.setCredentials(new AuthScope(AuthScope.ANY_HOST,
					AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthPolicy.NTLM), new NTCredentials(
					username, password, "", domain));
			this.authPref.add(AuthPolicy.NTLM);
		} else if ("spnego".equals(type)) {
			System.setProperty("java.security.auth.login.config", HttpClientManager.class
					.getResource("/resources/login.conf").toString());
			System.setProperty("javax.security.auth.useSubjectCredsOnly", "false");
			this.credentialsProvider.setCredentials(new AuthScope(AuthScope.ANY_HOST,
					AuthScope.ANY_PORT, AuthScope.ANY_REALM, AuthPolicy.SPNEGO),
					new UsernamePasswordCredentials("root", "root"));
			this.authPref.add(AuthPolicy.SPNEGO);
		}
	}

	/**
	 * Redirect strategy that checks we are still on the same domain
	 * 
	 * @see org.apache.http.impl.client.DefaultRedirectStrategy
	 */
	private class SameHostRedirectStrategy implements RedirectStrategy {
		public static final String REDIRECT_LOCATIONS = "http.protocol.redirect-locations";

		@Override
		public final boolean isRedirected(HttpRequest request, HttpResponse response,
				HttpContext context) {

			if (response == null) {
				throw new IllegalArgumentException("HTTP response may not be null");
			}

			int statusCode = response.getStatusLine().getStatusCode();
			switch (statusCode) {
			case HttpStatus.SC_MOVED_TEMPORARILY:
			case HttpStatus.SC_MOVED_PERMANENTLY:
			case HttpStatus.SC_TEMPORARY_REDIRECT:
			case HttpStatus.SC_SEE_OTHER:
				HttpUriRequest newRequest;
				try {
					newRequest = getRedirect(response, context, false);
					if (!newRequest.getURI().getHost().equals(
							((HttpUriRequest) ((RequestWrapper) request).getOriginal()).getURI()
									.getHost())) {
						return false;
					}
				} catch (ProtocolException e) {
					return false;
				}
				return true;
			default:
				return false;
			}
		}

		@Override
		public HttpUriRequest getRedirect(HttpRequest executedRequest, HttpResponse response,
				HttpContext context) throws ProtocolException {
			return getRedirect(response, context, true);
		}

		/**
		 * 
		 * @param response
		 *            the response received from the target server
		 * @param context
		 *            the context for the request execution
		 * @param handleCircular
		 *            check circular redirect
		 * @return redirected request
		 * @throws ProtocolException
		 *             on exception
		 */
		private HttpUriRequest getRedirect(HttpResponse response, HttpContext context,
				boolean handleCircular) throws ProtocolException {
			if (response == null) {
				throw new IllegalArgumentException("HTTP response may not be null");
			}
			// Get the location header to find out where to redirect to
			Header locationHeader = response.getFirstHeader("location");
			if (locationHeader == null) {
				// Got a redirect response, but no location header
				throw new ProtocolException("Received redirect response "
						+ response.getStatusLine() + " but no location header");
			}
			String location = locationHeader.getValue();

			URI uri;
			try {
				uri = new URI(location);
			} catch (URISyntaxException ex) {
				throw new ProtocolException("Invalid redirect URI: " + location, ex);
			}

			HttpParams params = response.getParams();
			// rfc2616 demands the location value be a complete URI
			// Location = "Location" ":" absoluteURI
			if (!uri.isAbsolute()) {
				if (params.isParameterTrue(ClientPNames.REJECT_RELATIVE_REDIRECT)) {
					throw new ProtocolException("Relative redirect location '" + uri
							+ "' not allowed");
				}
				// Adjust location URI
				HttpHost target = (HttpHost) context
						.getAttribute(ExecutionContext.HTTP_TARGET_HOST);
				if (target == null) {
					throw new IllegalStateException("Target host not available "
							+ "in the HTTP context");
				}

				HttpRequest request = (HttpRequest) context
						.getAttribute(ExecutionContext.HTTP_REQUEST);

				try {
					URI requestURI = new URI(request.getRequestLine().getUri());
					URI absoluteRequestURI = URIUtils.rewriteURI(requestURI, target, true);
					uri = URIUtils.resolve(absoluteRequestURI, uri);
				} catch (URISyntaxException ex) {
					throw new ProtocolException(ex.getMessage(), ex);
				}
			}

			if (handleCircular && params.isParameterFalse(ClientPNames.ALLOW_CIRCULAR_REDIRECTS)) {

				RedirectLocations redirectLocations = (RedirectLocations) context
						.getAttribute(REDIRECT_LOCATIONS);

				if (redirectLocations == null) {
					redirectLocations = new RedirectLocations();
					context.setAttribute(REDIRECT_LOCATIONS, redirectLocations);
				}

				URI redirectURI;
				if (uri.getFragment() != null) {
					try {
						HttpHost target = new HttpHost(uri.getHost(), uri.getPort(), uri
								.getScheme());
						redirectURI = URIUtils.rewriteURI(uri, target, true);
					} catch (URISyntaxException ex) {
						throw new ProtocolException(ex.getMessage(), ex);
					}
				} else {
					redirectURI = uri;
				}

				if (redirectLocations.contains(redirectURI)) {
					throw new CircularRedirectException("Circular redirect to '" + redirectURI
							+ "'");
				} else {
					redirectLocations.add(redirectURI);
				}
			}

			return new HttpGet(uri);
		}
	}
}
