package com.vulnit.projects.xss;

import com.vulnit.projects.xss.plugins.AttributeValuePlugin;
import com.vulnit.projects.xss.plugins.HrefSrcValuePlugin;
import com.vulnit.projects.xss.plugins.HtmlCommentPlugin;
import com.vulnit.projects.xss.plugins.Plugin;
import com.vulnit.projects.xss.plugins.RcDataPlugin;
import com.vulnit.projects.xss.plugins.ScriptComment;
import com.vulnit.projects.xss.plugins.ScriptValuePlugin;
import com.vulnit.projects.xss.plugins.TextPlugin;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;

/**
 * A manager for the plugins
 * 
 * @author rmillet
 * 
 */
public final class PluginManager {
	List<Plugin> plugins = new ArrayList<Plugin>();

	/**
	 * 
	 */
	public PluginManager() {
		plugins.clear();
		plugins.add(new RcDataPlugin());
		plugins.add(new TextPlugin());
		plugins.add(new AttributeValuePlugin());
		plugins.add(new HtmlCommentPlugin());
		plugins.add(new HrefSrcValuePlugin());
		plugins.add(new ScriptValuePlugin());
		plugins.add(new ScriptComment());
	}

	/**
	 * Execute each plugin on the current attribute of the {@link WebPage} until the payload
	 * injection is successful
	 * 
	 * @param webPage
	 *            The {@link WebPage} at its current state
	 * @param elements
	 *            The elements containing searchText
	 * @param searchText
	 *            The initial text injected
	 * @param prefix
	 *            prefix added before each payload
	 * @return The payload that was successful or <code>null</code>
	 */
	public String exec(WebPage webPage, Elements elements, String searchText, String prefix) {
		List<String> testedPayload = new ArrayList<String>();
		for (Plugin plugin : plugins) {
			String payload = plugin.getInitialPayload(elements, searchText);
			while (payload != null) {
				if (!testedPayload.contains(payload)) {
					try {
						Document document;

						if (!prefix.isEmpty()) {
							document = Jsoup.parse(webPage.getContent(prefix + payload));

							if (plugin.isInjectionSuccessful(document)) {
								return payload;
							}
						}

						document = Jsoup.parse(webPage.getContent(payload));
						if (plugin.isInjectionSuccessful(document)) {
							return payload;
						}
					} catch (IOException e) {
						System.err.println(e.getMessage());
					}
				}
				payload = plugin.getPayload();
			}
		}
		return null;
	}

}
