package com.vulnit.projects.xss;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;

/**
 * 
 * @author rmillet
 * 
 */
public final class WebPage {
	private final String baseUrl;
	private final List<NameValuePair> getData = new ArrayList<NameValuePair>();
	private final List<NameValuePair> postData = new ArrayList<NameValuePair>();

	private int currentParameter = -1;

	/**
	 * 
	 * @param url
	 *            The url of the WebPage
	 * @param postData
	 *            Post data of the request or <code>null</code>
	 * @throws URISyntaxException
	 *             When get or post parameters are not well-formatted
	 */
	public WebPage(String url, String postData) throws URISyntaxException {
		int pos = url.indexOf('?');
		if (pos != -1) {
			this.baseUrl = url.substring(0, pos);
		} else {
			this.baseUrl = url;
		}

		try {
			this.getData.addAll(URLEncodedUtils.parse(new URI(url), HTTP.UTF_8));
			if (postData != null) {
				this.postData.addAll(URLEncodedUtils.parse(new URI("http://0.0.0.0/?" + postData),
						HTTP.UTF_8));
			}
		} catch (URISyntaxException e) {
			if (e.getInput().startsWith("http://0.0.0.0/?")) {
				throw new URISyntaxException(e.getInput().substring(16), e.getReason());
			}
		}
	}

	/**
	 * Select the next parameter.
	 * 
	 * @return <code>true</code> if there are still parameters to inject
	 */
	public boolean hasNextParameter() {
		if (this.currentParameter + 1 < this.getData.size() + this.postData.size()) {
			++this.currentParameter;
			return true;
		}
		return false;
	}

	/**
	 * 
	 * @param currentParameterValue
	 *            Fetch the webpage with the current argument value replaced by
	 *            currentParameterValue
	 * @return A {@link String} containing the webpage response
	 * @throws IOException
	 *             on IO problems
	 */
	public String getContent(String currentParameterValue) throws IOException {
		List<NameValuePair> getDataTmp = new ArrayList<NameValuePair>();
		for (int i = 0; i < this.getData.size(); ++i) {
			if (i == this.currentParameter) {
				getDataTmp.add(new BasicNameValuePair(getData.get(i).getName(),
						currentParameterValue));
			} else {
				getDataTmp.add(getData.get(i));
			}
		}
		String uri = this.baseUrl;
		if (!getDataTmp.isEmpty()) {
			uri += "?" + URLEncodedUtils.format(getDataTmp, HTTP.UTF_8);
		}

		List<NameValuePair> postDataTmp = new ArrayList<NameValuePair>();
		for (int i = 0; i < this.postData.size(); ++i) {
			if (i == this.currentParameter - this.getData.size()) {
				postDataTmp.add(new BasicNameValuePair(postData.get(i).getName(),
						currentParameterValue));
			} else {
				postDataTmp.add(postData.get(i));
			}
		}

		HttpUriRequest request;
		if (!postDataTmp.isEmpty()) {
			HttpPost post = new HttpPost(uri);
			try {
				post.setEntity(new UrlEncodedFormEntity(postDataTmp, HTTP.UTF_8));
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
			request = post;
		} else {
			HttpGet get = new HttpGet(uri);
			request = get;
		}

		// Fetch content
		HttpEntity entity = HttpClientManager.getInstance().execute(request);
		if (entity != null) {
			String entityStr = EntityUtils.toString(entity);
			request.abort();
			return entityStr;
		} else {
			request.abort();
			throw new IOException("Entity is null");
		}
	}

	/**
	 * Return a string representing the current web-page with currentParameterValue defined for the
	 * current parameter
	 * 
	 * @param currentParameterValue
	 *            The value
	 * @return A string representing the URL with the post data
	 */
	public String toString(String currentParameterValue) {
		List<NameValuePair> getDataTmp = new ArrayList<NameValuePair>();
		for (int i = 0; i < this.getData.size(); ++i) {
			if (i == this.currentParameter) {
				getDataTmp.add(new BasicNameValuePair(getData.get(i).getName(),
						currentParameterValue));
			} else {
				getDataTmp.add(getData.get(i));
			}
		}
		String uri = this.baseUrl;
		if (!getDataTmp.isEmpty()) {
			uri += "?" + URLEncodedUtils.format(getDataTmp, HTTP.UTF_8);
		}

		List<NameValuePair> postDataTmp = new ArrayList<NameValuePair>();
		for (int i = 0; i < this.postData.size(); ++i) {
			if (i == this.currentParameter - this.getData.size()) {
				postDataTmp.add(new BasicNameValuePair(postData.get(i).getName(),
						currentParameterValue));
			} else {
				postDataTmp.add(postData.get(i));
			}
		}

		if (!postDataTmp.isEmpty()) {
			uri += " -PostData: " + URLEncodedUtils.format(postDataTmp, HTTP.UTF_8);
		}
		return uri;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof WebPage) {
			WebPage that = (WebPage) obj;
			return this.baseUrl.equals(that.baseUrl) && this.getData.size() == that.getData.size()
					&& this.postData.size() == that.postData.size()
					&& this.getData.containsAll(that.getData)
					&& this.postData.containsAll(that.postData);
		}
		return false;
	}

	@Override
	public int hashCode() {
		int hashCode = this.baseUrl.hashCode();
		hashCode = 31 * hashCode + this.getData.hashCode();
		hashCode = 31 * hashCode + this.postData.hashCode();
		return hashCode;
	}

	/**
	 * 
	 * @return The port of the WebPage
	 */
	public int getPort() {
		try {
			URI uri = new URI(this.baseUrl);
			if (uri.getPort() != -1) {
				return uri.getPort();
			} else if (uri.getScheme().equals("http")) {
				return 80;
			} else if (uri.getScheme().equals("https")) {
				return 443;
			}
		} catch (URISyntaxException e) {

			e.printStackTrace();
		}

		return -1;
	}

	/**
	 * 
	 * @return the current attribute name
	 */
	public String getAttributeName() {
		if (this.currentParameter < 0
				|| this.currentParameter >= this.getData.size() + this.postData.size()) {
			return "";
		}

		if (this.currentParameter < this.getData.size()) {
			return this.getData.get(this.currentParameter).getName();
		}

		return this.postData.get(this.currentParameter - this.getData.size()).getName();
	}
}
