package com.vulnit.projects.xss.plugins;

import com.vulnit.projects.js.SimpleJsParser;
import com.vulnit.projects.js.elements.JsComment;
import com.vulnit.projects.js.elements.JsElement;
import com.vulnit.projects.js.elements.JsElements;
import com.vulnit.projects.js.elements.JsText;

import java.util.LinkedList;
import java.util.List;

import org.jsoup.nodes.DataNode;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Node;
import org.jsoup.select.Elements;

/**
 * Handle comment in script tags
 * 
 * @author rmillet
 * 
 */
public final class ScriptComment extends Plugin {
	private final List<String> comments = new LinkedList<String>();

	@Override
	public String getInitialPayload(Elements elements, String searchText) {
		this.comments.clear();
		for (Element element : elements) {
			if (element.tagName().equals("script")) {
				for (Node node : element.childNodes()) {
					if (node instanceof DataNode) {
						DataNode dataNode = (DataNode) node;
						if (dataNode.toString().toLowerCase().contains(searchText)) {
							handleJs(dataNode.toString(), searchText);
						}
					}
				}
			}
		}
		return getPayload();
	}

	/**
	 * 
	 * @param data
	 *            Data string
	 * @param searchText
	 *            The text injected
	 */
	private void handleJs(String data, String searchText) {
		JsElements jsElements = SimpleJsParser.parse(data);
		for (JsElement jsElement : jsElements) {
			if (jsElement.getText().contains(searchText)) {
				if (jsElement instanceof JsComment) {
					JsComment jsComment = (JsComment) jsElement;

					String end;
					if (jsComment.isMultiline()) {
						end = "*/";
					} else {
						end = "\n";
					}

					if (!this.comments.contains(end)) {
						this.comments.add(end);
					}
				}
			}
		}
	}

	@Override
	public String getPayload() {
		if (!this.comments.isEmpty()) {
			String quote = this.comments.remove(0);
			return quote + "(alert(" + NumTag + "))";
		}
		return null;
	}

	@Override
	public boolean isInjectionSuccessful(Document document) {
		Elements e = document.getAllElements();
		for (Element element : e) {
			if (element.tagName().equals("script")) {
				for (Node node : element.childNodes()) {
					if (node instanceof DataNode) {
						DataNode dataNode = (DataNode) node;
						JsElements jsElements = SimpleJsParser.parse(dataNode.toString());
						for (JsElement jsElement : jsElements) {
							if (jsElement.getText().contains("(alert(" + NumTag + "))")
									&& jsElement instanceof JsText) {
								return true;
							}
						}
					}
				}
			}
		}
		return false;
	}
}
