package com.vulnit.projects.xss.plugins;

import com.vulnit.projects.js.SimpleJsParser;
import com.vulnit.projects.js.elements.JsElement;
import com.vulnit.projects.js.elements.JsElements;
import com.vulnit.projects.js.elements.JsString;
import com.vulnit.projects.js.elements.JsText;

import java.util.LinkedList;
import java.util.List;

import org.jsoup.nodes.Attribute;
import org.jsoup.nodes.DataNode;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.nodes.Node;
import org.jsoup.select.Elements;

/**
 * Find XSS in on* values (eg: &lt;tag onmouseover="injected_value"&gt;, or &lt;a
 * href="javascript:injected_value"&gt; or &lt;script&gt;)
 * 
 * @author rmillet
 * 
 */
public final class ScriptValuePlugin extends Plugin {
	private final List<String> quotes = new LinkedList<String>();
	private boolean isUnquoted = false;

	@Override
	public String getInitialPayload(Elements elements, String searchText) {
		this.quotes.clear();
		for (Element element : elements) {
			for (Attribute attr : element.attributes()) {
				if ((attr.getKey().startsWith("on") || (attr.getKey().equals("src") && attr
						.getValue().matches("(?is)[a-zA-Z]+script:.*")))
						&& attr.getValue().contains(searchText)) {
					handleJs(attr.getValue(), searchText);
				}
			}

			if (element.tagName().equals("script")) {
				for (Node node : element.childNodes()) {
					if (node instanceof DataNode) {
						DataNode dataNode = (DataNode) node;
						if (dataNode.toString().toLowerCase().contains(searchText)) {
							handleJs(dataNode.toString(), searchText);
						}
					}
				}
			}
		}
		return getPayload();
	}

	/**
	 * 
	 * @param data
	 *            Data string
	 * @param searchText
	 *            The text injected
	 */
	private void handleJs(String data, String searchText) {
		JsElements jsElements = SimpleJsParser.parse(data);
		for (JsElement jsElement : jsElements) {
			if (jsElement.getText().contains(searchText)) {
				if (jsElement instanceof JsString) {
					JsString jsString = (JsString) jsElement;
					String quote = String.valueOf(jsString.getType());
					if (!this.quotes.contains(quote)) {
						this.quotes.add(quote);
					}
				} else if (jsElement instanceof JsText) {
					if (!this.quotes.contains("")) {
						this.quotes.add("");
					}
				}
			}
		}
	}

	@Override
	public String getPayload() {
		if (!this.quotes.isEmpty()) {
			String quote = this.quotes.remove(0);
			if (quote.isEmpty()) {
				isUnquoted = true;
				return "(alert(" + NumTag + "))";
			}
			isUnquoted = false;
			return quote + ";alert(" + quote + Plugin.Tag + quote + ");" + quote;
		}
		return null;
	}

	@Override
	public boolean isInjectionSuccessful(Document document) {
		Elements e = document.getAllElements();

		for (Element element : e) {
			for (Attribute attr : element.attributes()) {
				if (attr.getKey().startsWith("on")
						|| (attr.getKey().equals("src") && attr.getValue().matches(
								"(?is)[a-zA-Z]+script:.*"))) {
					if (testSuccess(attr.getValue())) {
						return true;
					}
				}
			}

			if (element.tagName().equals("script")) {
				for (Node node : element.childNodes()) {
					if (node instanceof DataNode) {
						DataNode dataNode = (DataNode) node;
						if (testSuccess(dataNode.toString())) {
							return true;
						}
					}
				}
			}
		}
		return false;
	}

	/**
	 * 
	 * @param data
	 *            Data string
	 * @return <code>true</code> if the injection is successful
	 */
	private boolean testSuccess(String data) {
		JsElements jsElements = SimpleJsParser.parse(data);
		boolean isAlert = false;
		boolean isTag = false;
		for (JsElement jsElement : jsElements) {
			if (isUnquoted) {
				// Payload contains a number
				if (jsElement.getText().contains("(alert(" + NumTag + "))")
						&& jsElement instanceof JsText) {
					return true;
				}
				continue;
			}

			// Payload contains a quoted string
			if (isAlert) {
				if (isTag) {
					if (jsElement.getText().startsWith(");") && jsElement instanceof JsText) {
						return true;
					}
				} else if (jsElement.getText().equals(Tag) && jsElement instanceof JsString) {
					isTag = true;
					continue;
				}

				isTag = false;
				isAlert = false;
			}
			if (jsElement.getText().endsWith("alert(") && jsElement instanceof JsText) {
				isAlert = true;
			}
		}
		return false;
	}
}
